from collections import deque, defaultdict

def find_guarded_warehouses(n, m, k, roads, guards):
    # Построим граф
    graph = defaultdict(list)

    for a, b in roads:
        graph[a].append(b)
        graph[b].append(a)

    # Множество для хранения охраняемых складов
    guarded_warehouses = set()

    # Для каждого охранника и его выносливости проводим BFS
    for p, h in guards:
        # BFS
        queue = deque([(p, 0)])  # (текущий склад, расстояние)
        visited = set()
        visited.add(p)

        while queue:
            current_warehouse, distance = queue.popleft()
            guarded_warehouses.add(current_warehouse)

            if distance < h:  # Если текущая дистанция меньше выносливости
                for neighbor in graph[current_warehouse]:
                    if neighbor not in visited:
                        visited.add(neighbor)
                        queue.append((neighbor, distance + 1))

    # Получаем результат
    guarded_list = sorted(guarded_warehouses)
    return len(guarded_list), guarded_list

# Чтение входных данных
n, m, k = map(int, input().strip().split())
roads = [tuple(map(int, input().strip().split())) for _ in range(m)]
guards = [tuple(map(int, input().strip().split())) for _ in range(k)]

# Получение охраняемых складов
guarded_count, guarded_warehouses = find_guarded_warehouses(n, m, k, roads, guards)

# Вывод результата
print(guarded_count)
if guarded_count > 0:
    print(' '.join(map(str, guarded_warehouses)))
else:
    print()